;;; timestamp.el --- smart timestamp interface for emacs.

;; Copyright (C) 1997-2000 Free Software Foundation, Inc.

;; Author: Kevin A. Burton (burton@openprivacy.org)
;; Maintainer: Kevin A. Burton (burton@openprivacy.org)
;; Location: http://relativity.yi.org
;; Keywords: time stamp timestamp
;; Version: 1.0.0

;; This file is [not yet] part of GNU Emacs.

;; This program is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free Software
;; Foundation; either version 2 of the License, or any later version.
;;
;; This program is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License along with
;; this program; if not, write to the Free Software Foundation, Inc., 59 Temple
;; Place - Suite 330, Boston, MA 02111-1307, USA.

;;; Commentary:

;; functions for inserting a timestamp and an associated regexp for fontlock

;;; Install:

;; (require 'timestamp) in your .emacs

;;; History:
;; 

;; Tue May 08 2001 03:03 AM (burton@relativity.yi.org): 

;;; Code:
(defface timestamp-face '((t (:foreground "YellowGreen" :italic t)))
  "Face used to show timestamps.")

(defun timestamp-insert()
  "Insert a username based timestamp"
  (interactive)

  (insert (concat (format-time-string "%a %h %d %Y %I:%M %p")
                  " ("
                  user-mail-address
                  "): ")))

(defun timestamp-font-lock()
  "Adds a font-lock for buffers that user opens"
  (interactive)

  ;;font-lock which starts with a day... ends with a colon.


  (font-lock-add-keywords nil '(("\\(Mon\\|Tue\\|Wed\\|Thu\\|Fri\\|Sat\\|Sun\\) [A-Z][a-z]+ [0-9]+ [0-9]+ [0-9:]+ [AP]M ([a-z@.]+):" 
                                 0 'timestamp-face t))))

  
(add-hook 'font-lock-mode-hook 'timestamp-font-lock)

(global-set-key "\C-ct" 'timestamp-insert)

(provide 'timestamp)

;;; timestamp.el ends here
