;;; w3c-comment.el --- Add w3c comments to XML/HTML/SGML modes.

;; $Id: w3c-comment.el,v 1.4 2002/01/18 11:14:57 burton Exp $

;; Copyright (C) 1997-2000 Free Software Foundation, Inc.

;; Author: Kevin A. Burton (burton@openprivacy.org)
;; Maintainer: Kevin A. Burton (burton@openprivacy.org)
;; Location: http://relativity.yi.org
;; Keywords:
;; Version: 1.0.0

;; This file is [not yet] part of GNU Emacs.

;; This program is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free Software
;; Foundation; either version 2 of the License, or any later version.
;;
;; This program is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License along with
;; this program; if not, write to the Free Software Foundation, Inc., 59 Temple
;; Place - Suite 330, Boston, MA 02111-1307, USA.

;;; Commentary:

;; This is a hack for adding XML/HTML/SGML W3C style comment font-lock to
;; existing modes.  Some of the SGML emacs stuff wasn't correctly font-locking
;; this and it was getting on my nerves to I wrote this.  Note that the existing
;; sgml-mode only font-locks comments which are on line.
;;
;; Basically it uses a function to match between <!-- --> since it is basically
;; impossible to write a regexp that does this effectively.


;;; TODO:

;; Sat Sep 29 2001 01:32 PM (burton@relativity.yi.org): 
;;
;; This doesn't work uner Emacs21... why is this?  It looks like something to do
;; with font-lock narrowing the region and setting the limit to a small value.
;;
;;  - This works in Emacs 21.3 from CVS.

;; In a lot of situations this make it a PAIN to deal with.  Rewrite this so
;; that it is compatible with Emacs 21 and 20.x.

;; - Wed Apr 11 13:12:44 2001 (burton@relativity.yi.org): if we enter an empty
;;   comment beginning... <!-- ... then we should comment until the end of the
;;   buffer.  Don't know if we should do this.  
;;

;;; History:
;; 
;; - Wed Apr 11 13:01:58 2001 (burton@relativity.yi.org): change the laxmatch in
;;   font-lock to overwrite all fontification
;;
;; - Wed Apr 11 13:04:02 2001 (burton@relativity.yi.org): Need to support nested
;; comments.  NOTE:  We should not enable this feature. XML does not support this.
;;
;; Example:
;;
;;  <!--
;;  comment level one
;;     <!--  comment level two -->
;;  -->
;;

;;; Code:
(defvar w3c-comment-start "<!--" "W3C comment starter.")

(defvar w3c-comment-end "-->" "W3C comment end.")

(defun w3c-comment-search-forward(limit)
  "Used by font-lock and matches the font-lock function definition (set
match-data correctly and return non nil if it worked).  Search forward for a W3C
comment marker '<!--' and then search for a comment end '-->'"
  (interactive
   (list nil))
  
  ;;WARNING: Emacs21 narrows on me... we can't have this!

  ;;NOTE: there was no real way to set the match-data without a hack.  If the
  ;;format ever changes in the format ever changes in the future this will
  ;;break.  
  (let(begin end)
    (if (search-forward w3c-comment-start nil t)
        (progn
          (setq begin (match-beginning 0))
          (goto-char begin)

          (if (re-search-forward w3c-comment-end nil t)
              (progn
                (setq end (match-end 0))

;;                 (let((marker-begin (make-marker))
;;                      (marker-end (make-marker)))

;;                   (set-marker marker-begin begin (current-buffer))
;;                   (set-marker marker-end end (current-buffer))
                    
;;                   (set-match-data (list marker-begin marker-end)))

                ;;
                (add-text-properties begin end '(comment t face font-lock-comment-face) (current-buffer))

;;                                   (goto-char begin)
;;                                   (push-mark nil t t)
;;                                   (goto-char end)

                  
                t)
            nil))
      nil)))

(defun w3c-comment-add-mode(mode)
  
  (font-lock-add-keywords mode '((w3c-comment-search-forward
                                  (0 'font-lock-comment-face t)))))

;;no go through and add all the modes
(w3c-comment-add-mode 'html-mode)
(w3c-comment-add-mode 'sgml-mode)
(w3c-comment-add-mode 'xae-mode)
(w3c-comment-add-mode 'xml-mode)

(provide 'w3c-comment)

;;; w3c-comment.el ends here
